<?php
  include_once ("./controller/IController.php");
  include_once ("./util/DbUtil.php");
  include_once ("./model/Manga.php");
  include_once ("./model/Publisher.php");
  include_once ("./model/Language.php");
  include_once ("./model/Genre.php");
  include_once ("./model/Author.php");

  class MangaEditController implements IController {
    private function loadReferences() {
      $dao = DbUtil::getDao();

      $query = "SELECT publisher_id, publisher_name from manga_publisher ORDER BY publisher_name;";
      $result = $dao->query($query);
      $GLOBALS['publishers'] = $this->resultToPublisher($result);

      $query = "SELECT language_id, language_name from manga_language ORDER BY language_name;";
      $result = $dao->query($query);
      $GLOBALS['languages'] = $this->resultToLanguage($result);

      $query = "SELECT genre_id, genre_name from manga_genre ORDER BY genre_name;";
      $result = $dao->query($query);
      $GLOBALS['genres'] = $this->resultToGenre($result);

      $query = "SELECT author_id, author_name_romanji, author_name from manga_author ORDER BY author_name_romanji;";
      $result = $dao->query($query);
      $GLOBALS['authors'] = $this->resultToAuthor($result);
    }

    public function execute(&$errors) {
      $id = $_REQUEST['id'];
      $manga = null;

      $this->loadReferences();

      $dao = DbUtil::getDao();
      if (isSet($id) && $id != "") {
        $query = "SELECT title_id, title_name, title_original_name, title_original_name_romanji, title_rating, title_volume_count, " .
                 "title_synopsis, publisher_id, language_id, author_id " .
                "FROM manga_title WHERE title_id = $id;";
        $mangaResult = $dao->query($query);

        $query = "SELECT genre_id FROM manga_title_genre WHERE title_id = $id;";
        $genresResult = $dao->query($query);

        $manga = new Manga($mangaResult[0], $genresResult);
      } else {
        $manga = new Manga();
      }
      return $manga;
    }

    public function save(&$errors) {
      $insert = TRUE;
      $dialect = DbUtil::getDialect();

      $id = $_REQUEST['id'];
      $name = $dialect->escapeString($_REQUEST['name']);
      $originalName = $dialect->escapeString($_REQUEST['originalName']);
      $originalNameRomanji = $dialect->escapeString($_REQUEST['originalNameRomanji']);
      $volumeCount = $_REQUEST['volumeCount'];
      $synopsis = $dialect->escapeString($_REQUEST['synopsis']);
      $publisher = $_REQUEST['publisher'];
      $language = $_REQUEST['language'];
      $author = $_REQUEST['author'];
      $rating = $_REQUEST['rating'];

      if ($publisher == "") {
        $publisher = 'NULL';
      }
      if ($language == "") {
        $language = 'NULL';
      }
      if ($author == "") {
        $author = 'NULL';
      }

      $manga = $this->execute($errors);
      $manga->setId($id);
      $manga->setName($_REQUEST['name']);
      $manga->setOriginalName($_REQUEST['originalName']);
      $manga->setOriginalNameRomanji($_REQUEST['originalNameRomanji']);
      $manga->setVolumeCount($_REQUEST['volumeCount']);
      $manga->setPublisher($_REQUEST['publisher']);
      $manga->setLanguage($_REQUEST['language']);
      $manga->setGenres($_REQUEST['genres']);
      $manga->setAuthor($_REQUEST['author']);
      $manga->setSynopsis($_REQUEST['synopsis']);
      $manga->setRating($_REQUEST['rating']);

      if ($name == "") {
        $errors->addError(new Error("name", AbstractLocale::getInstance()->getTranslation("error_required_name")));
      }

      if ($volumeCount == "") {
        $errors->addError(new Error("volumeCount", AbstractLocale::getInstance()->getTranslation("error_required_volume_count")));
      }

      // Test si c'est un nombre entier > 0
      if (is_numeric($volumeCount) == FALSE || strval (intval($volumeCount)) !== $volumeCount) {
        $errors->addError(new Error("volumeCount", AbstractLocale::getInstance()->getTranslation("error_volume_number_format")));
      } else if (intval($volumeCount) <= 0) {
        $errors->addError(new Error("volumeCount", AbstractLocale::getInstance()->getTranslation("error_volume_number_positive")));
      }

      if ($errors->hasErrors() == false) {
        $query = "";
        if (isSet($id) && $id != "") {
          $query = "UPDATE manga_title " .
                  "SET title_name = '$name', " .
                  "title_original_name = '$originalName', " .
                  "title_original_name_romanji = '$originalNameRomanji', " .
                  "title_volume_count = $volumeCount, " .
                  "title_rating = $rating, " .
                  "title_synopsis = '$synopsis', " .
                  "publisher_id = $publisher, " .
                  "language_id = $language, " .
                  "author_id = $author " .
                  "WHERE title_id = $id;";
           $insert = FALSE;
        } else {
          $query = "INSERT INTO manga_title (title_name, title_original_name, title_original_name_romanji, title_volume_count, title_rating, " .
                    " title_synopsis, publisher_id, language_id, author_id)" .
                    " values ('$name', '$originalName', '$originalNameRomanji', $volumeCount, $rating, '$synopsis', $publisher, $language, $author);";
          $insert = TRUE;
        }

        $dao = DbUtil::getDao();
        try {
          $dao->queryUpdate($query);

          if ($insert === TRUE) {
            $id = $dao->getLastId();
          }

          $query = "DELETE FROM manga_title_genre WHERE title_id = $id;";
          $dao->queryUpdate($query);

          foreach ($_REQUEST['genres'] as $genreId) {
            $query = "INSERT INTO manga_title_genre (title_id, genre_id) VALUES ($id, $genreId);";
            $dao->queryUpdate($query);
          }

          // adapte le nombre de volumes
          if ($insert === FALSE) {
            $query = "DELETE FROM manga_volume WHERE title_id = $id AND (volume_nr < 1 OR volume_nr > $volumeCount);";
            $dao->queryUpdate($query);
          }

          // ajoute les volumes manquants
          for($volumeNr = 1; $volumeNr <= $volumeCount; $volumeNr++) {
            $query = "SELECT COUNT(*) as count FROM manga_volume WHERE title_id = $id and volume_nr = $volumeNr;";
            $genresResult = $dao->query($query);

            if ($genresResult[0]->count == 0) {
              $query = "INSERT INTO manga_volume (volume_nr, title_id) VALUES ($volumeNr, $id);";
              $dao->queryUpdate($query);
            }
          }

          $manga->setId($id);
          $dao->commit();
        } catch(Exception $exception) {
          $errors->addError(new Error(Error::$GLOBAL_ERROR, AbstractLocale::getInstance()->getTranslation("error_save")));
          $dao->rollback();
        }
      }
      return $manga;
    }

    public function volume(&$errors) {
      $model = null;

      $manga = $this->save($errors);
      $_REQUEST['id'] = $manga->getId();

      if ($errors->hasErrors()) {
        $this->loadReferences();
        $model = $manga;
      }
      return $model;
    }

    private function resultToPublisher($result) {
      $publishers = array();

      foreach($result as $publisherRow) {
        $publisher = new Publisher($publisherRow);
        array_push($publishers, $publisher);
      }
      return $publishers;
    }

    private function resultToLanguage($result) {
      $languages = array();

      foreach($result as $languageRow) {
        $language = new Language($languageRow);
        array_push($languages, $language);
      }
      return $languages;
    }

    private function resultToGenre($result) {
      $genres = array();

      foreach($result as $genreRow) {
        $genre = new Genre($genreRow);
        array_push($genres, $genre);
      }
      return $genres;
    }

    private function resultToAuthor($result) {
      $authors = array();

      foreach($result as $authorRow) {
        $author = new Author($authorRow);
        array_push($authors, $author);
      }
      return $authors;
    }
  }
?>