<?php
  include_once ("./controller/IController.php");
  include_once ("./util/DbUtil.php");
  include_once ("./util/Utils.php");
  include_once ("./model/Volume.php");

  class VolumeEditController implements IController {
    public function execute(&$errors) {
      $id = $_REQUEST['id'];
      $volume = null;

      $dao = DbUtil::getDao();

      if (isSet($id) && $id != "") {
        $query = "SELECT volume_id, volume_nr, volume_owned, volume_release, volume_delayed, volume_isbn, title_id " .
                "FROM manga_volume WHERE volume_id = $id;";
        $volumeResult = $dao->query($query);

        $volume = new Volume($volumeResult[0]);

        $query = "SELECT volume_image_id FROM manga_volume_images WHERE volume_id = $id;";
        $imageIds = $dao->query($query);

        $ids = array();
        foreach ($imageIds as $imageId) {
          array_push($ids, $imageId->volume_image_id);
        }

        $volume->setImageIds($ids);
      }
      return $volume;
    }

    public function save(&$errors) {
      $id = $_REQUEST['id'];
      $owned = 0;
      $delayed = 0;
      $isbn = "NULL";
      $dialect = DbUtil::getDialect();
      
      if (isSet($_REQUEST['owned'])) {
        $owned = $_REQUEST['owned'];
      }
      if (isSet($_REQUEST['delayed'])) {
        $delayed = $_REQUEST['delayed'];
      }
      $releaseDate = $_REQUEST['releaseDate'];
      if (isSet($_REQUEST['isbn'])) {
        $isbn = "'" . $dialect->escapeString($_REQUEST['isbn']) . "'";
      }
      $titleId = $_REQUEST['titleId'];

      $volume = $this->execute($errors);
      $volume->setOwned($owned);
      $volume->setDelayed($delayed);
      $volume->setReleaseDate($releaseDate);
      $volume->setIsbn($_REQUEST['isbn']);

      if ($owned != TRUE) {
        $owned = 0;
      }

      if ($delayed != TRUE) {
        $delayed = 0;
      }

      if(Utils::checkDate($releaseDate) == FALSE) {
        $errors->addError(new Error("releaseDate", AbstractLocale::getInstance()->getTranslation("error_invalid_date")));
      } else {
        $timestamp = Utils::dateToTimestamp($releaseDate);
        $releaseDate = DbUtil::getDialect()->toDate($timestamp);
      }

      $query = "UPDATE manga_volume " .
              "SET volume_owned = $owned, " .
              "volume_delayed = $delayed, " .
              "volume_release = $releaseDate, " .
              "volume_isbn = $isbn " .
              "WHERE volume_id = $id;";

      $dao = DbUtil::getDao();

      try {
        $dao->queryUpdate($query);
        $dao->commit();
      } catch(Exception $exception) {
        $errors->addError(new Error(Error::$GLOBAL_ERROR, AbstractLocale::getInstance()->getTranslation("error_save")));
        $dao->rollback();
      }
      return $volume;
    }

    public function upload(&$errors) {
        return $this->save($errors);
    }

    public function saveUpload(&$errors) {
        $mimes = array('image/pjpeg', 'image/jpeg', 'image/x-png', 'image/png', 'image/gif');
    
        $volume = $this->execute($errors);

        if(!isset($_FILES['uploadfile']) || !isset($_FILES['uploadfile']['name']) || $_FILES['uploadfile']['name'] == '') {
            //$errors->addError(new Error(Error::$GLOBAL_ERROR, "erreur"));
        } else {
            if (is_uploaded_file($_FILES['uploadfile']['tmp_name'])) {
              if(in_array($_FILES['uploadfile']['type'], $mimes)) {
            	$imageStr = file_get_contents($_FILES['uploadfile']['tmp_name']);
            	$size = getImageSize($_FILES['uploadfile']['tmp_name']);
            	$imageStr = addslashes ($imageStr);
            	$width = $size[0];
            	$height = $size[1];
                
                $query = "INSERT INTO manga_volume_images (volume_id, volume_image_mime, volume_image_name, volume_image_data) " .
                         "VALUES (" . $_REQUEST['id'] . ", '" . $_FILES['uploadfile']['type'] . "', '" . 
                         $_FILES['uploadfile']['name'] . "', '" . $imageStr . "');";

                $dao = DbUtil::getDao();

                try {
                    $dao->queryUpdate($query);
                    $dao->commit();
                } catch(Exception $exception) {
                    $errors->addError(new Error(Error::$GLOBAL_ERROR, AbstractLocale::getInstance()->getTranslation("error_save")));
                    $dao->rollback();
                }
              } else {
                $errors->addError(new Error(Error::$GLOBAL_ERROR, AbstractLocale::getInstance()->getTranslation("error_invalid_mime")));
              }
            } else {
              $errors->addError(new Error(Error::$GLOBAL_ERROR, AbstractLocale::getInstance()->getTranslation("error_not_uploaded")));
            }
        }
        return $volume;
    }

    public function deleteUpload(&$errors) {
      $massDelete = $_REQUEST['massDelete'];
      $result = null;

      if (count($massDelete) > 0) {
        $dao = DbUtil::getDao();

        try {
          foreach($massDelete as $id) {
            $dao->queryUpdate("DELETE FROM manga_volume_images WHERE volume_image_id = $id;");
          }
        } catch(Exception $exception) {
          $errors->addError(new Error(Error::$GLOBAL_ERROR, AbstractLocale::getInstance()->getTranslation("error_delete")));
          $dao->rollback();
          $result = $this->execute($errors);
        }
        $dao->commit();
      }
      return $result;
    }
  }
?>