<?php
  include_once("./admin/themeconfig.php");
  include_once("./util/Errors.php");
  include_once("./util/HTML.php");
  include_once("./util/DbUtil.php");
  include_once("./dao/DAOException.php");
  include_once("./locale/AbstractLocale.php");

  $dispatcher = new Dispatcher();
  $dispatcher->dispatch();
  
  function __autoload($class_name) {
    if (substr($class_name, 0, 3) == "DAO") {
      include_once("./dao/" . $class_name . ".php");
    } else if (strripos($class_name, "Dialect") != FALSE) {
      include_once("./dialect/" . $class_name . ".php");
    } else if(substr($class_name, 0, 6) == "Locale") {
      include_once "./locale/" . $class_name . '.php';
    }
  }

  class Dispatcher {
    public function dispatch() {
      session_start();
      
      global $theme;
    
      if(isSet($_REQUEST['newTheme'])) {
        $_SESSION['theme'] = $_REQUEST['newTheme'];
      }
    
      if (isSet($_SESSION['theme'])) {
        $theme = $_SESSION['theme'];
      }
      

      if (isSet($_REQUEST['do'])) {
        $actionName = $_REQUEST['do'];

        $fileData = file_get_contents("config.xml");
        $config = simplexml_load_string($fileData);

        $action = $this->getAction($config, $actionName);

        if (isSet($action)) {
          $attributes = $action->attributes();

          $controllerName = $attributes['controller'];
          $commandName = $attributes['commandName'];

          if(isSet($_REQUEST['methodController'])) {
            $method = $_REQUEST['methodController'];
          }

          if (isSet($method)) {
            $formViewAttributes = $this->getFormView($action, $method)->attributes();
          } else {
            $method = "execute";
            if(isSet($action->formView[0])) {
                $formViewAttributes = $action->formView[0]->attributes();
            } else {
                $formViewAttributes = null;
            }
          }

          $errors = new Errors();

          $daoException = FALSE;
          try {
            $model = $this->executeController($config, $controllerName, $method, $errors);

            if (isSet($model)) {
              $GLOBALS["$commandName"] = $model;
            }
            $GLOBALS["errors"] = $errors;

          } catch(DAOException $daoe) {
            $daoException = TRUE;
            $_GLOBALS['daoException'] = $daoe;
          }

          DbUtil::disconnect();

          if ($daoException === FALSE) {
            if ($errors->hasErrors()) {
              $this->showView($config, $action->formView[0]->attributes(), null);
            } else if(isSet($formViewAttributes)){
              $this->showView($config, $formViewAttributes, $model);
            }
          } else {
            include('./error.php');
          }
        } else {
		include('./view/page/invalidAction.tpl');
        }
      } else {
        header("location: index.php?do=mangaList");
      }
    }

    private function getAction($config, $actionName) {
        $action = null;

        for($i = 0; isSet($config->actions[0]->action[$i]) && !isSet($action); $i++) {
          $attributes = $config->actions[0]->action[$i]->attributes();
          if ($actionName == $attributes['path']) {
            $action = $config->actions[0]->action[$i];
          }
        }
        return $action;
    }

    private function executeController($config, $controllerName, $method, &$errors) {
      $attributes = $config->paths[0]->controller[0]->attributes();
      $controllerPath = $attributes['path'];

      include_once("$controllerPath/$controllerName.php");

      $name = $controllerName . "";
      $controller = new $name();

      $reflectionClass = new ReflectionClass("$name");
      $reflectionMethod = $reflectionClass->getMethod($method);
      return $reflectionMethod->invoke($controller, $errors);
    }

    private function showView($config, $formView, $model) {
      $attributes = $config->paths[0]->view[0]->attributes();
      $viewPath = $attributes['path'];

      $viewName = $formView['name'];
      $viewType = $formView['type'];

      if (!isSet($viewType) || $viewType=="forward") {
        include ("$viewPath/$viewName");
      } else if (isSet($viewType) && $viewType=="redirect") {
        $url = 'location: http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).'/'. $viewName;
        header(HTML::createURL($url));
      }
    }

    private function getFormView($action, $method) {
      $formView = null;

      for($i = 0; isSet($action->method[$i]) && !isSet($formView); $i++) {
        $attributes = $action->method[$i]->attributes();
        if ($method == $attributes['name']) {
          $formView = $action->method[$i]->formView[0];
        }
      }

      return $formView;
    }
  }
?>