<?php
  include_once("./locale/ILocale.php");

  abstract class AbstractLocale implements ILocale {
    private static $instance;

    protected $translations = Array();

    static function getInstance() {
      if(!isSet(AbstractLocale::$instance)) {
        $codeLanguage = AbstractLocale::getLanguageCode();
        $locale = "Locale_" . $codeLanguage;
        AbstractLocale::$instance = new $locale();
        AbstractLocale::$instance->init();
      }
      return AbstractLocale::$instance;
    }

    // Retourne la langue utilise pour les traductions
    private static function getLanguageCode() {
      $acceptLanguage = getenv("HTTP_ACCEPT_LANGUAGE");
      $usedLanguage;

      $last=substr($acceptLanguage,-1);
      if($last != ",") {
        $acceptLanguage .= ",";
      }
      $acceptLanguage = preg_replace("/ /", "", $acceptLanguage);
      $acceptLanguage = preg_replace("/;(.).*?,/", ",", $acceptLanguage);

      $languages = explode(",", $acceptLanguage);
      array_pop($languages);

      // reformate les codes
      $newLanguages = array();
      foreach($languages as $language) {
        if(strpos($language, "-") === FALSE) {
          $lng = $language;
        } else {
          list($lng, $country) = explode("-", $language);
        }
        $newLanguage = $lng;
        if (isSet($country)) {
          $newLanguage .= "_" . strtoupper($country);
        }
        array_push($newLanguages, $newLanguage);
      }
      $languages = $newLanguages;

      // recherche le fichier correspondant au code langue
      foreach($languages as $language) {
        if(!isSet($usedLanguage) && file_exists("./locale/Locale_" . $language . ".php")) {
          $usedLanguage = $language;
        }
      }

      // recherche le fichier correpondant au code langue seul (ne tient pas compte du code pays)
      if(!isSet($usedLanguage)) {
        foreach($languages as $language) {
          $onlyLanguage = substr($language, 0, 2);

          if(!isSet($usedLanguage) && $onlyLanguage != $language && file_exists("./locale/locale_" . $onlyLanguage . ".php")) {
            $usedLanguage = $onlyLanguage;
          }
        }
      }

      // dfaut anglais
      if(!isSet($usedLanguage)) {
        $usedLanguage = "en";
      }

      return $usedLanguage;
    }

    function getTranslation($code) {
      if (isSet($this->translations[$code])) {
        $val = $this->translations[$code];
      } else {
        $className = get_class($this);
        $val = "???" . $code . "-" . substr($className, 7) . "???" ;
      }
      return $val;
    }

    protected abstract function init();
  }
?>