CREATE TABLE phpmymanga.manga_publisher
(
  publisher_id INTEGER NOT NULL auto_increment,
  publisher_name VARCHAR(20) NOT NULL,
  publisher_website VARCHAR(255) NULL,
  PRIMARY KEY (publisher_id),
  UNIQUE KEY publisher_name (publisher_name)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_genre
(
  genre_id INTEGER NOT NULL auto_increment,
  genre_name VARCHAR(20) NOT NULL,
  PRIMARY KEY (genre_id),
  UNIQUE KEY genre_name (genre_name)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_language
(
  language_id INTEGER NOT NULL auto_increment,
  language_name VARCHAR(20) NOT NULL,
  PRIMARY KEY (language_id),
  UNIQUE KEY language_name (language_name)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_author
(
  author_id INTEGER NOT NULL auto_increment,
  author_name_romanji VARCHAR(50) NOT NULL,
  author_name VARCHAR(50) NULL,
  PRIMARY KEY (author_id),
  UNIQUE KEY author_name (author_name)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_title
(
   title_id INTEGER NOT NULL auto_increment,
   title_name VARCHAR(100) NOT NULL,
   title_original_name VARCHAR(100),
   title_original_name_romanji VARCHAR(100),
   title_volume_count INTEGER NOT NULL,
   title_synopsis LONGTEXT NULL,
   title_rating INTEGER NOT NULL DEFAULT 0,
   publisher_id INTEGER NULL,
   language_id INTEGER NULL,
   author_id INTEGER NULL,
   PRIMARY KEY (title_id),
   FOREIGN KEY (publisher_id) REFERENCES phpmymanga.manga_publisher(publisher_id),
   FOREIGN KEY (language_id) REFERENCES phpmymanga.manga_language(language_id),
   FOREIGN KEY (author_id) REFERENCES phpmymanga.manga_author(author_id)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_title_genre
(
   title_id INTEGER NOT NULL,
   genre_id INTEGER NOT NULL,
   PRIMARY KEY (title_id, genre_id),
   FOREIGN KEY (title_id) REFERENCES phpmymanga.manga_title(title_id),
   FOREIGN KEY (genre_id) REFERENCES phpmymanga.manga_genre(genre_id)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_volume
(
   volume_id INTEGER NOT NULL auto_increment,
   volume_nr INTEGER NOT NULL,
   volume_owned BOOL NOT NULL DEFAULT false,
   volume_release DATE,
   volume_delayed BOOL NOT NULL DEFAULT false,
   volume_isbn VARCHAR(13),
   title_id INTEGER NOT NULL,
   PRIMARY KEY (volume_id),
   UNIQUE KEY title_volume (title_id, volume_nr),
   FOREIGN KEY (title_id) REFERENCES phpmymanga.manga_title(title_id)
) ENGINE=INNODB;

CREATE TABLE phpmymanga.manga_volume_images
(
   volume_image_id INTEGER NOT NULL auto_increment,
   volume_id INTEGER NOT NULL,
   volume_image_mime VARCHAR(30) NOT NULL,
   volume_image_name VARCHAR(255) NOT NULL,
   volume_image_data LONGBLOB NOT NULL,
   PRIMARY KEY (volume_image_id),
   FOREIGN KEY (volume_id) REFERENCES phpmymanga.manga_volume(volume_id)
) ENGINE=INNODB;
