<?php
  class BreadCrumbElement {
    private $parent = null;
    private $action = null;
    private $name = null;
    private $params = null;

    public function __construct($action, $name, $parent, $params) {
      $this->action = $action;
      $this->name   = $name;
      $this->parent = $parent;
      $this->params = $params;
    }

    public function getAction() {
      return $this->action;
    }

    public function getName() {
      return $this->name;
    }

    public function getParent() {
      return $this->parent;
    }
    
    public function getLink() {
      $link = $this->getAction();
      if ($this->params != null) {
        $link .= "&amp;" . $this->params;
      }
      return $link;
    }
  }

  class BreadCrumb {
    private $elements;

    private function init() {
      $this->elements = array();

      $elementRef = new BreadCrumbElement("referenceList", "breadcrumb.references", null, null);
      $this->elements[$elementRef->getAction()] = $elementRef;

      $elementList = new BreadCrumbElement("publisherList", "breadcrumb.publishers", $elementRef, null);
      $this->elements[$elementList->getAction()] = $elementList;

      $elementEdit = new BreadCrumbElement("publisherEdit", "breadcrumb.publisher", $elementList, null);
      $this->elements[$elementEdit->getAction()] = $elementEdit;

      $elementList = new BreadCrumbElement("genreList", "breadcrumb.genres", $elementRef, null);
      $this->elements[$elementList->getAction()] = $elementList;

      $elementEdit = new BreadCrumbElement("genreEdit", "breadcrumb.genre", $elementList, null);
      $this->elements[$elementEdit->getAction()] = $elementEdit;

      $elementList = new BreadCrumbElement("languageList", "breadcrumb.languages", $elementRef, null);
      $this->elements[$elementList->getAction()] = $elementList;

      $elementEdit = new BreadCrumbElement("languageEdit", "breadcrumb.language", $elementList, null);
      $this->elements[$elementEdit->getAction()] = $elementEdit;
      
      $elementList = new BreadCrumbElement("authorList", "breadcrumb.authors", $elementRef, null);
      $this->elements[$elementList->getAction()] = $elementList;

      $elementEdit = new BreadCrumbElement("authorEdit", "breadcrumb.author", $elementList, null);
      $this->elements[$elementEdit->getAction()] = $elementEdit;

      $elementManga = new BreadCrumbElement("mangaList", "breadcrumb.mangas", null, null);
      $this->elements[$elementManga->getAction()] = $elementManga;
      
      $elementEdit = new BreadCrumbElement("mangaEdit", "breadcrumb.manga", $elementManga, "id={titleId}");
      $this->elements[$elementEdit->getAction()] = $elementEdit;
      
      $elementList = new BreadCrumbElement("volumeList", "breadcrumb.volumes", $elementEdit, "titleId={titleId}");
      $this->elements[$elementList->getAction()] = $elementList;
      
      $elementEdit = new BreadCrumbElement("volumeEdit", "breadcrumb.volume", $elementList, "id={id}&amp;titleId={titleId}");
      $this->elements[$elementEdit->getAction()] = $elementEdit;
      
      $elementImage = new BreadCrumbElement("volumeUpload", "breadcrumb.images", $elementEdit, null);
      $this->elements[$elementImage->getAction()] = $elementImage;
      
      $elementSearch = new BreadCrumbElement("searchCriteria", "breadcrumb.searchCriteria", null, null);
      $this->elements[$elementSearch->getAction()] = $elementSearch;
      
      $elementList = new BreadCrumbElement("searchList", "breadcrumb.searchList", $elementSearch, null);
      $this->elements[$elementList->getAction()] = $elementList;
      
      $elementReleases = new BreadCrumbElement("releases", "breadcrumb.releases", null, null);
      $this->elements[$elementReleases->getAction()] = $elementReleases;
    }

    public function createBreadCrumb() {
      $breadcrumb = "";
      $this->init();
      
      $actionName = $_REQUEST['do'];
      if (isSet($actionName)) {
        $element = $this->elements[$actionName];
        
        if (isSet($element)) {
          $breadcrumb = AbstractLocale::getInstance()->getTranslation($element->getName());
          
          while ($element->getParent() != null) {
            $element = $element->getParent();
            $link = "<a href=\"index.php?do=" . HTML::createURL($element->getLink()) . "\">" .
                    AbstractLocale::getInstance()->getTranslation($element->getName()) . "</a>";
            $breadcrumb = $link . " > " . $breadcrumb;
          }
        }
      }
      return $breadcrumb;
    }
  }
?>